# Sample one control for every case

library( plyr )

# --- Configuration --- #
.db <- "~/projects"
basedir <- file.path(.db,"misc","bootLR_Applied")
source( file.path( basedir, "source", "configure_R.R" ) )

# --- Code --- #

# -- Load and clean data -- #

testData <- read.csv( file.path( intdir, "bootLR - Article Abstraction Data Tables - Test Data 20160630.csv" ) )
load( file.path( intdir, "combinedSearch.Rdata" ) ) # articleDF: Search results from PubMed and Scopus (without filtering down to "100%"" results)
stopifnot( all( testData$ID %in% articleDF$pmid ) ) # confirm that ID in testData is just pmid in the combined search (tricky since there's Scopus)
controlData <- articleDF[ ( ! articleDF$pmid %in% testData$ID ) & ( !articleDF$match ), ]

# -- Sample -- #

# - Create lists of journal-years among cases to use for stratified sampling - #
caseJournalYear <- tolower( with( testData, paste( Journal, Year, sep = "~" ) ) )
controlData$journalYear <- tolower( with( controlData, paste( journal, year, sep = "~" ) ) )

# Confirm there are at least as many controls as cases for every journal-year
journalYearTab <- merge( 
  as.data.frame( table( caseJournalYear ) ),
  as.data.frame( table( controlData$journalYear[ controlData$journalYear %in% caseJournalYear ] ) ),
  by.x = "caseJournalYear", by.y = "Var1", all.x = TRUE, all.y = FALSE
)
stopifnot( all( with( journalYearTab, Freq.y >= Freq.x ) ) ) 

# - Simple random sample - #
set.seed( 2387858 )
nSRS <- 150
articleDFmatchingYears <- articleDF[ articleDF$year %in% testData$Year, ]
cat("There were ", nrow(articleDFmatchingYears), " articles in the years that we pulled cases from, which forms our sampling frame.")
srsIdx <- sample.int( nrow( articleDFmatchingYears ), size = nSRS )
allArticleSrs <- articleDFmatchingYears[ srsIdx, ]
colnames( allArticleSrs )[ colnames( allArticleSrs ) == "pmid" ] <- "ID"
allArticleSrs$AlreadyAbstractedCase <- allArticleSrs$ID %in% testData$ID
cat("Of those, ", sum(allArticleSrs$AlreadyAbstractedCase), "have already been used as cases, which is", round(sum(allArticleSrs$AlreadyAbstractedCase)/nSRS,3)*100,"% of our sample. This is about what we'd expect (compare to the", round( nrow(testData) / nrow(articleDFmatchingYears), 3 ) * 100, "% that is the number of cases divided by the number of all articles in those years.")
write.csv( allArticleSrs, file.path( intdir, "SimpleRandomSampleArticles.csv" ), row.names = FALSE )

# - Simple random sample, second pull- #
  # We didn't get quite as many as expected, so pulling another 60 just to be safe
set.seed( 8096816 )
nSRS <- 69 # this was just calibrated so we get 60 after excluding those that were already pulled in round 1
articleDFmatchingYears <- articleDF[ articleDF$year %in% testData$Year, ]
cat("There were ", nrow(articleDFmatchingYears), " articles in the years that we pulled cases from, which forms our sampling frame.")
srs2Idx <- sample.int( nrow( articleDFmatchingYears ), size = nSRS )
srs2Idx <- srs2Idx[ !srs2Idx %in% srsIdx ]
allArticleSrs <- articleDFmatchingYears[ srs2Idx, ]
colnames( allArticleSrs )[ colnames( allArticleSrs ) == "pmid" ] <- "ID"
allArticleSrs$AlreadyAbstractedCase <- allArticleSrs$ID %in% testData$ID
cat("Of those, ", sum(allArticleSrs$AlreadyAbstractedCase), "have already been used as cases, which is", round(sum(allArticleSrs$AlreadyAbstractedCase)/nSRS,3)*100,"% of our sample. This is about what we'd expect (compare to the", round( nrow(testData) / nrow(articleDFmatchingYears), 3 ) * 100, "% that is the number of cases divided by the number of all articles in those years.")
write.csv( allArticleSrs, file.path( intdir, "SimpleRandomSampleArticles_Batch2.csv" ), row.names = FALSE )

# - Simple random sample, third pull- #
# Just a few more needed, so pulling another 50 just to be safe
set.seed( 224774 )
nSRS3 <- 61 # this was just calibrated so we get 50 after excluding those that were already pulled in rounds 1 & 2
articleDFmatchingYears <- articleDF[ articleDF$year %in% testData$Year, ]
cat("There were ", nrow(articleDFmatchingYears), " articles in the years that we pulled cases from, which forms our sampling frame.")
srs3Idx <- sample.int( nrow( articleDFmatchingYears ), size = nSRS3 )
srs3Idx <- srs3Idx[ (!srs3Idx %in% srsIdx) & (!srs3Idx %in% srs2Idx) ]
allArticleSrs3 <- articleDFmatchingYears[ srs3Idx, ]
colnames( allArticleSrs3 )[ colnames( allArticleSrs3 ) == "pmid" ] <- "ID"
allArticleSrs3$AlreadyAbstractedCase <- allArticleSrs3$ID %in% testData$ID
cat("Of those, ", sum(allArticleSrs$AlreadyAbstractedCase), "have already been used as cases, which is", round(sum(allArticleSrs$AlreadyAbstractedCase)/nSRS,3)*100,"% of our sample. This is about what we'd expect (compare to the", round( nrow(testData) / nrow(articleDFmatchingYears), 3 ) * 100, "% that is the number of cases divided by the number of all articles in those years.")
write.csv( allArticleSrs, file.path( intdir, "SimpleRandomSampleArticles_Batch3.csv" ), row.names = FALSE )


# - Simple random sample, fourth pull- #
# Found to have a few duplicates, pulling 20 more
set.seed( 7119123 )
nSRS4 <- 27 # this was just calibrated so we get 20 after excluding those that were already pulled in rounds 1 & 2 & 3
articleDFmatchingYears <- articleDF[ articleDF$year %in% testData$Year, ]
cat("There were ", nrow(articleDFmatchingYears), " articles in the years that we pulled cases from, which forms our sampling frame.")
srs4Idx <- sample.int( nrow( articleDFmatchingYears ), size = nSRS4 )
srs4Idx <- srs4Idx[ (!srs4Idx %in% srsIdx) & (!srs4Idx %in% srs2Idx) & (!srs4Idx %in% srs3Idx) ]
allArticleSrs4 <- articleDFmatchingYears[ srs4Idx, ]
colnames( allArticleSrs4 )[ colnames( allArticleSrs4 ) == "pmid" ] <- "ID"
allArticleSrs4$AlreadyAbstractedCase <- allArticleSrs4$ID %in% testData$ID
cat("Of those, ", sum(allArticleSrs$AlreadyAbstractedCase), "have already been used as cases, which is", round(sum(allArticleSrs$AlreadyAbstractedCase)/nSRS4,3)*100,"% of our sample. This is about what we'd expect (compare to the", round( nrow(testData) / nrow(articleDFmatchingYears), 3 ) * 100, "% that is the number of cases divided by the number of all articles in those years.")
write.csv( allArticleSrs4, file.path( intdir, "SimpleRandomSampleArticles_Batch4.csv" ), row.names = FALSE )


# - Simple random sample, fifth pull- #
# Found to have a few duplicates, pulling 20ish more
set.seed( 8564808 )
nSRS5 <- 27 
articleDFmatchingYears <- articleDF[ articleDF$year %in% testData$Year, ]
cat("There were ", nrow(articleDFmatchingYears), " articles in the years that we pulled cases from, which forms our sampling frame.")
srs5Idx <- sample.int( nrow( articleDFmatchingYears ), size = nSRS5 )
srs5Idx <- srs5Idx[ (!srs5Idx %in% srsIdx) & (!srs5Idx %in% srs2Idx) & (!srs5Idx %in% srs3Idx)  & (!srs5Idx %in% srs4Idx) ]
allArticleSrs5 <- articleDFmatchingYears[ srs5Idx, ]
colnames( allArticleSrs5 )[ colnames( allArticleSrs5 ) == "pmid" ] <- "ID"
allArticleSrs5$AlreadyAbstractedCase <- allArticleSrs5$ID %in% testData$ID
cat("Of those, ", sum(allArticleSrs$AlreadyAbstractedCase), "have already been used as cases, which is", round(sum(allArticleSrs$AlreadyAbstractedCase)/nSRS5,3)*100,"% of our sample. This is about what we'd expect (compare to the", round( nrow(testData) / nrow(articleDFmatchingYears), 3 ) * 100, "% that is the number of cases divided by the number of all articles in those years.")
write.csv( allArticleSrs5, file.path( intdir, "SimpleRandomSampleArticles_Batch5.csv" ), row.names = FALSE )


# - Stratified Sample - #
# This allows us to see whether usage of likelihood ratios and confidence intervals differs between 100% s/s papers and non-100% s/s papers

set.seed( 7624408 )

matchedControls <- ddply( controlData[ controlData$journalYear %in% caseJournalYear, ], 
  .( journal, year ),
  function( dat, testData ) {
     cat( dat$journal[1], dat$year[1], "\n" )
     caseDat <- testData[ testData$Journal == dat$journal[1] & testData$Year == dat$year[1], ]
     sampledControls <- dat[ sample.int( n = nrow(dat), size = nrow(caseDat) ), ]
     stopifnot( nrow( sampledControls ) > 0 & nrow( caseDat ) > 0 & length( nrow( sampledControls ) ) == length( nrow( caseDat ) ) )
     sampledControls$matchedCaseID <- sample( caseDat$ID, size = nrow( caseDat ) )
     sampledControls
  }, 
  testData = testData
)

write.csv( matchedControls, file.path( intdir, "matchedControls.csv" ), row.names = FALSE )


