# Parse PubMed XML files and restrict to those with sens/spec = 100%

suppressPackageStartupMessages( library(XML) )
suppressPackageStartupMessages( library(taRifx) )
suppressPackageStartupMessages( library(dplyr) )
suppressPackageStartupMessages( library(stringr) )

# --- Initialize --- #

.db <- "~/Dropbox"
basedir <- file.path(.db,"misc","bootLR_Applied")
datadir <- file.path( basedir, "data" )

source( file.path( basedir, "source", "configure_R.R" ) )

# --- Code --- #

# -- Load -- #

pm <- xmlInternalTreeParse( file.path( datadir, "DatabaseSearches", "20160126_PubMedSearch19journals_allSSarticles.xml" ) )
pmList <- xmlToList( pm )

# -- Parse into abstracts -- #

pmDF <- taRifx::stack.list( lapply( pmList, function(paper) { 
  pmid <- paper$MedlineCitation$PMID$text
  if( length(pmid) != 1 )  pmid <- NA
  doi <- paper$MedlineCitation$Article$ELocationID$text
  if( length(doi) != 1 )  doi <- NA
  abstract <- paste0( unname( unlist( paper$MedlineCitation$Article$Abstract ) ), collapse = "\n" )
  if( length(abstract) != 1 )  abstract <- NA
  journal <- paper$MedlineCitation$MedlineJournalInfo$MedlineTA
  if( length(journal) != 1 )  journal <- NA
  year <- paper$MedlineCitation$Article$Journal$JournalIssue$PubDate$Year
  if( length(year) != 1 )  year <- paper$MedlineCitation$Article$ArticleDate$Year
  if( length(year) != 1 )  year <- paper$MedlineCitation$DateCompleted$Year
  if( length(year) != 1 )  year <- NA
  name <- paper$MedlineCitation$Article$ArticleTitle
  if( length(name) != 1 )  name <- NA
  firstauth <- paper$MedlineCitation$Article$AuthorList[[1]]$LastName
  if( length(firstauth) != 1 )  firstauth <- NA
  # Keywords
  kw <- sapply( paper$MedlineCitation$KeywordList, function(x) x[[1]] )
  kw <- kw[ names(kw) == "Keyword" ]
  kw <- paste0( kw, collapse = " ~ " )
  if( length( kw ) != 1 )  kw <- NA
  # Combine and return result
  dateAccessed = Sys.Date()
  data.frame( pmid = pmid, doi = doi, firstauth = firstauth, name = name, abstract = abstract, journal = journal, year = year, keywords = kw, dateAccessed = dateAccessed )
} ) )

# -- Flag papers meeting search criteria -- #

# First set of exclusions: No abstract available in PubMed
pmDF$exclusion <- NA_character_
pmDF$exclusion[ pmDF$abstract == "" ] <- "Exclusion 1 - No abstract in database"
# If it contains "100%" or "0%"
pmDF$match_100pct <- grepl( "100[%]", gsub( " *", "", pmDF$abstract ) )
pmDF$match_0pct <- grepl( "^0[%]", gsub( " *", "", pmDF$abstract ) ) | grepl( "[a-zA-Z]0[%]", gsub( " *[.-]*", "", pmDF$abstract ) )
# If it contains two non-year integers equal to each other e.g. 27/27
# nums <- str_extract_all( pmDF$abstract, "[0-9]+[.]?[0-9]*[%]?" ) # Leave the decimals and percents in, we'll grab them later
# selRepeatNums <- sapply( nums, function(x) {
#   x <- x[ !grepl( "[%]", x ) ] # drop all percentages
#   x <- x[ !grepl( "[0-9]+[.][0-9]+", x ) ] # drop all non-integers
#   x <- sub( "([0-9]+)[.]$", "\\1", x ) # delete periods from ends of sentences
#   x <- x[ !grepl( "^20[0-9][0-9]$", x ) ] # drop all 20XX years
#   x <- x[ !grepl( "^19[0-9][0-9]$", x ) ] # drop all 19XX years
#   if( length(x) > 0 ) {
#     res <- any( rle( x )$lengths>1 ) 
#   } else {
#     res <- FALSE
#   }
#   if(res)    browser()
#   res
# })
# pmDF$abstract[ selRepeatNums ]


# -- Export final result -- #

write.csv( pmDF, file = file.path( intdir, "PubMed_cleaned.csv" ), row.names = FALSE )

save( nSampledCases, nPotentialCases, file = file.path( intdir, "FinalResults_cleanPubMed" ) )
